@@ -10,17 +10,18 @@ from pathlib import Path
 
 class CommonData(InputData):
     # waste Materials
-    WasteMaterials = ['FoodWaste', 'Compost', 'Digestate',
+    WasteMaterials = ['FoodWaste', 'Compost', 'ADLiquids', 'ADSolids',
                       'MSW', 'CombustionResiduals', 'CompostResiduals',
-                      'ContaminatedSoil',
+                      'ContaminatedSoil', 'C_DWaste', 'AFFF',
                       'LFLeachate', 'ContaminatedWater', 'ContactWater', 'WWTEffluent',
-                      'RawWWTSolids', 'DewateredWWTSolids', 'DriedWWTSolids', 'WWTSolids',
+                      'RawWWTSolids', 'DewateredWWTSolids', 'DriedWWTSolids', 'WWTScreenRejects',
                       'SCWOSteam', 'SCWOSlurry',
-                      'SpentGAC', 'ROConcentrate']
+                      'SpentGAC', 'ROConcentrate', 'StabilizedSoil',
+                      'SpentIER']
 
     # PFAS Index
-    PFAS_Index = ['Summed PFAS', 'PFOA', 'PFOS', 'PFAS#4', 'PFAS#5', 'PFAS#6',
-                  'PFAS#7', 'PFAS#8', 'PFAS#9', 'PFAS#10']
+    PFAS_Index = ['PFOA', 'PFOS', 'PFBA', 'PFPeA', 'PFHxA', 'PFHpA',
+                  'PFNA', 'PFDA', 'PFBS', 'PFHxS']
 
     def __init__(self, input_data_path=None):
         if input_data_path:
