@@ -19,7 +19,10 @@ from .ProcessModelsMetaData import ProcessModelsMetaData
 
 
 class Project():
-    def __init__(self, Inventory, CommonData, ProcessModels=None):
+    def __init__(self, Inventory, CommonData, ProcessModels=None, pop_up=None):
+
+        self.pop_up = pop_up
+
         self.Inventory = Inventory
 
         self.CommonData = CommonData
@@ -68,20 +71,33 @@ class Project():
         self.InputFlow = InputFlow
 
         ProcessSet = set()
+        ProcessSetPrim = set()
+        ProcessSetSec = set()
+        ProcessSetTer = set()
+        ProcessSetQua = set()
         # Input flow
         for P in self.WasteTreatment[self.InputFlow.FlowType]:
             ProcessSet.add(P)
+            ProcessSetPrim.add(P)
             # Intermediate products (Level 1)
             for Product1 in self.ProcessModels[P]['Model'].ProductsType:
                 # Add the processes for treatment of level 1 products
                 for PP in self.WasteTreatment[Product1]:
                     ProcessSet.add(PP)
+                    ProcessSetSec.add(PP)
                     # Intermediate products (Level 2)
                     for Product2 in self.ProcessModels[PP]['Model'].ProductsType:
                         # Add the processes for treatment of level 2 products
                         for PPP in self.WasteTreatment[Product2]:
                             ProcessSet.add(PPP)
-        return(ProcessSet)
+                            ProcessSetTer.add(PPP)
+                            # Intermediate products (Level 3)
+                            for Product3 in self.ProcessModels[PPP]['Model'].ProductsType:
+                                # Add the processes for treatment of level 3 products
+                                for PPPP in self.WasteTreatment[Product3]:
+                                    ProcessSet.add(PPPP)
+                                    ProcessSetQua.add(PPPP)
+        return(ProcessSet, ProcessSetPrim, ProcessSetSec, ProcessSetTer, ProcessSetQua)
 
     def set_process_set(self, ProcessSet):
         self.ProcessSet = ProcessSet
@@ -131,35 +147,44 @@ class Project():
                     raise ValueError('Sum of the fractions for {} is not 1'.format(key))
         self.FlowParams = FlowParams
 
-    def setup_network(self):
+    def setup_network(self, Cut_Off=0.001):
         self.Inventory.clear()
-        self.CuttOff = sum(self.InputFlow.PFAS.values) * 0.01
+        self.CuttOff = sum(self.InputFlow.PFAS.values) * Cut_Off
         product = {self.InputFlow.FlowType: self.InputFlow}
-        Project.calc(product, self.InputFlow.FlowType, self.ProcessModels, self.FlowParams, self.Inventory, self.CuttOff, self._ProcessNameRef)
+        Project.calc(product, self.InputFlow.FlowType, self.ProcessModels, self.FlowParams, self.Inventory, self.CuttOff,
+                     self._ProcessNameRef, self.WasteTreatment, pop_up=self.pop_up)
 
     @staticmethod
-    def calc(product, source, processmodel, FlowParams, Inventory, CuttOff, ProcessNameRef):
+    def calc(product, source, processmodel, FlowParams, Inventory, CuttOff, ProcessNameRef, Treatment_options, pop_up=None):
         for prdct in product:
             if FlowParams[prdct]:
                 flows = split(InputFlow=product[prdct], **FlowParams[prdct])
                 for prcs, flw in flows.items():
-                    if sum(flw.PFAS.values) > CuttOff:
-                        if source not in ProcessNameRef:
-                            Inventory.add(prdct, source, prcs, flw)
-                        else:
-                            Inventory.add(prdct, source, prcs, flw)
+                    if np.isnan(sum(flw.PFAS.values)):
+                        msg = """ The PFAS flow for {} stream is nan! check the related input data and process model ({}).""".format(prdct, source)
+                        if pop_up:
+                            pop_up('PFAS Flow Warning!', msg, 'Warning')
+                        raise ValueError(msg)
+                    elif sum(flw.PFAS.values) > CuttOff:
+                        Inventory.add(prdct, source, prcs, flw)
                         processmodel[ProcessNameRef[prcs]]['Model'].calc(flw)
                         product_2 = processmodel[ProcessNameRef[prcs]]['Model'].products()
-                        Project.calc(product_2, prcs, processmodel, FlowParams, Inventory, CuttOff, ProcessNameRef)
+                        Project.calc(product_2, prcs, processmodel, FlowParams, Inventory, CuttOff, ProcessNameRef, Treatment_options, pop_up=pop_up)
             else:
-                # Change the warning to exception when all the process models are done
-                warnings.warn("No process is defined for {} treatment".format(prdct))
+                if sum(product[prdct].PFAS.values) > CuttOff:
+                    # Change the warning to exception when all the process models are done
+                    msg = "No process is defined for {} treatment! \nSelect one of the following processes: \n".format(prdct)
+                    for j in Treatment_options[prdct]:
+                        msg += processmodel[j]['Name'] + '\n'
+                    if pop_up:
+                        pop_up('Treatment Network Warning!', msg, 'Warning')
+                    warnings.warn(msg)
 
     def setup_MC(self, InputFlow_object, seed=None):
         self.InputFlow_object = InputFlow_object
-        self.InputFlow_object.setup_MC(seed=None)
+        self.InputFlow_object.setup_MC(seed=seed)
         for p in self.ProcessSet:
-            self.ProcessModels[p]['Model'].setup_MC(seed=None)
+            self.ProcessModels[p]['Model'].setup_MC(seed=seed)
 
     def MC_Next(self):
         NewInputData = []
@@ -173,10 +198,10 @@ class Project():
         self.Inventory.clear()
         self.InputFlow_object.calc()
         product = {self.InputFlow_object.Inc_flow.FlowType: self.InputFlow_object.Inc_flow}
-        Project.calc(product, 'Start', self.ProcessModels, self.FlowParams, self.Inventory, self.CuttOff, self._ProcessNameRef)
+        Project.calc(product, 'Start', self.ProcessModels, self.FlowParams, self.Inventory, self.CuttOff, self._ProcessNameRef, self.WasteTreatment)
         return(NewInputData)
 
-    def MC_Run(self, n, signal=None):
+    def MC_Run(self, n, TypeOfPFAS='All', signal=None):
         # Print warning only first time
         warnings.simplefilter('once', UserWarning)
 
@@ -185,7 +210,7 @@ class Project():
         MC_results = []
         for i in range(n):
             NewInputData = self.MC_Next()
-            MC_results.append((i, NewInputData, self.Inventory.report()))
+            MC_results.append((i, NewInputData, self.Inventory.report(TypeOfPFAS)))
             if signal:
                 if i in progress_steps:
                     progress += 5
@@ -233,10 +258,11 @@ class Project():
                 self.Inventory.clear()
                 self.InputFlow_object.calc()
                 product = {self.InputFlow_object.Inc_flow.FlowType: self.InputFlow_object.Inc_flow}
-                Project.calc(product, 'Start', self.ProcessModels, self.FlowParams, self.Inventory, self.CuttOff, self._ProcessNameRef)
+                Project.calc(product, 'Start', self.ProcessModels, self.FlowParams, self.Inventory, self.CuttOff,
+                             self._ProcessNameRef, self.WasteTreatment)
 
                 # Store Results
-                SA_results.append((i, [(Paramter, val)], self.Inventory.report(TypeOfPFAS)))
+                SA_results.append((i, [((Category, Paramter), val)], self.Inventory.report(TypeOfPFAS)))
 
         except Exception as e:
             print('Error while running the SA: \n')
@@ -255,11 +281,13 @@ class Project():
             ..note:: Make sure that the directory containing the dot executable is on your systems path.
 
             """
+            # Set the color for starting waste material
+            self._NtwkrNodeColor[self.InputFlow.FlowType] = 'chartreuse2'
             # Initialize PFAS treatment network
             self.network = graphviz.Digraph(name='Network', filename='Network.gv', format='png', engine='dot')
             self.network.graph_attr['rankdir'] = 'LR'
             self.network.attr(dpi='300')
-            self.network.attr(size='4,8')
+            self.network.attr(size='5,5')
             for x in self._NtwkrNode:
                 self.network.node(x, shape=self._NtwkrNodeShape[x], fillcolor=self._NtwkrNodeColor[x], style='filled', width='1.2')
 
@@ -286,28 +314,31 @@ class Project():
         color_link = []
 
         # List of colors: https://flaviocopes.com/rgb-color-codes/
-        edge_color_dict = {'CompFeed': (202, 255, 112),  # darkolivegreen1  #CAFF70
-                           'CompFin': (110, 139, 61),  # darkolivegreen4	#6E8B3D
-                           'CompRes': (118, 238, 0),  # chartreuse2	#76EE00
-                           'Digestate': (124, 252, 0),  # lawn green	#7CFC00
+        edge_color_dict = {'FoodWaste': (202, 255, 112),  # darkolivegreen1  #CAFF70
+                           'Compost': (110, 139, 61),  # darkolivegreen4	#6E8B3D
+                           'CompostResiduals': (118, 238, 0),  # chartreuse2	#76EE00
+                           'ADSolids': (124, 252, 0),  # lawn green	#7CFC00
                            'MSW': (139, 101, 8),  # darkgoldenrod4	#8B6508
-                           'C&DWaste': (238, 232, 205),  # cornsilk2	#EEE8CD
-                           'MedWaste': (220, 20, 60),  # crimson	#DC143C
-                           'MRFRes': (189, 183, 107),  # darkkhaki	#BDB76B
+                           'C_DWaste': (238, 232, 205),  # cornsilk2	#EEE8CD
+                           'ADLiquids': (220, 20, 60),  # crimson	#DC143C
+                           'SpentGAC': (189, 183, 107),  # darkkhaki	#BDB76B
                            'ContaminatedSoil': (128, 138, 135),  # coldgrey	#808A87
-                           'CombRes': (169, 169, 169),  # darkgray	#A9A9A9
+                           'CombustionResiduals': (169, 169, 169),  # darkgray	#A9A9A9
                            'ContactWater': (142, 229, 238),  # cadetblue2	#8EE5EE
                            'ContaminatedWater': (0, 0, 139),  # dark blue	#00008B
                            'LFLeachate': (0, 0, 255),  # blue	#0000FF
-                           'DewWWTSol': (128, 128, 128),  # gray / grey	#808080
-                           'DryWWTSol': (192, 192, 192),  # silver	#C0C0C0
-                           'WWTSol': (112, 128, 144),  # slate gray	#708090
-                           'SCWO_Steam': (123, 104, 238),  # medium slate blue	#7B68EE
-                           'SCWO_Slurry': (72, 61, 139),  # dark slate blue	#483D8B
-                           'SpentGAC': (138, 43, 226),  # blue violet	#8A2BE2
+                           'DewateredWWTSolids': (128, 128, 128),  # gray / grey	#808080
+                           'DriedWWTSolids': (192, 192, 192),  # silver	#C0C0C0
+                           'RawWWTSolids': (112, 128, 144),  # slate gray	#708090
+                           'ROConcentrate': (123, 104, 238),  # medium slate blue	#7B68EE
+                           'StabilizedSoil': (72, 61, 139),  # dark slate blue	#483D8B
                            'ROConc': (70, 130, 180),  # steel blue	#4682B4
                            'RunOff': (0, 255, 255)}  # aqua	#00FFFF
 
+        for i in Data.loc['Flow_name']:
+            if i not in edge_color_dict:
+                edge_color_dict[i] = (np.random.randint(0, 200), np.random.randint(0, 200), np.random.randint(0, 150))
+
         index = 0
         for i in Data.loc['Source']:
             if i not in label_dict:
@@ -358,7 +389,7 @@ class Project():
                            font_size=16,
                            hoverlabel=dict(font_size=14))
         data = go.Sankey(valueformat=".3s",
-                         valuesuffix="μg/year",
+                         valuesuffix="μg",
                          node=node,
                          link=link)
         fig = go.Figure(data=[data], layout=layout)
@@ -370,7 +401,7 @@ class Project():
         store_data['font_size'] = 16
         store_data['hoverlabel'] = dict(font_size=14)
         store_data['valueformat'] = ".3s"
-        store_data['valuesuffix'] = "μg/year"
+        store_data['valuesuffix'] = "μg"
         store_data['node'] = node
         store_data['link'] = link
 
